/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.entity.emotion;

import java.util.Random;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.db.file.node.NodeManager;
import re.jpayet.mentdb.core.entity.concentration.ConcentrationManager;
import re.jpayet.mentdb.core.entity.link.LinkManager;
import re.jpayet.mentdb.core.entity.relation.RelationManager;
import re.jpayet.mentdb.core.entity.thought.ThoughtManager;
import re.jpayet.mentdb.core.entity.word.WordManager;
import re.jpayet.mentdb.server.Start;
import re.jpayet.mentdb.tools.Misc;

public class EmotionManager {
	
	//Show all emotions
	@SuppressWarnings("unchecked")
	public static JSONArray showAll(String cuser) throws Exception {
		
		//Initialization
		JSONArray result = new JSONArray();
		
		JSONArray emotions = RelationManager.showTabLinks(cuser, "["+LinkManager.get_value("@[r_fr_?_est_une_émotion]")+" 2/3]", "L", ConcentrationManager.getConcentrationDepth("C[relation]"));
		
		//Parse all emotion relation tab links
		for(int i=0;i<emotions.size();i++) {
			
			if (i==ConcentrationManager.getConcentrationDepth("C[emotion]")) break; 
			
			//Get the current emotion relation
			String emotionRelationTabLink = emotions.get(i)+"";
			
			String relationId = Misc.atom(emotionRelationTabLink, 1, " ").substring(1);
			
			JSONArray words = RelationManager.showWords(cuser, relationId);
			
			JSONArray thoughts = RelationManager.showThoughtNodes(cuser, ""+RelationManager.showThoughtNodes(cuser, relationId).get(0));
			
			String thought = thoughts.get(1)+"";
			JSONObject bd = Record.getNode(thought);
			
			JSONArray a = (JSONArray) words.get(0);
			
			((JSONObject) a.get(0)).put("percent", bd.get("percent"));
			
			result.add(a);
			
		}
		
		return result;
		
	}
	
	//Create a new emotion
	public static String create(String cuser, String emotion, String lang, int theLevel, String type, boolean extreme, String parentThought, int parentAnLevel) throws Exception {
		
		if (type!=null) type=type.toLowerCase();
		
		//Check parameters
		if (Misc.lrtrim(emotion).equals("")) {
			throw new Exception("Sorry, the emotion is required."); 
		}
		if (type==null || (!type.equals("positive") && !type.equals("negative") && !type.equals("neutral"))) {
			throw new Exception("Sorry, the emotion type is not valid (positive, negative or neutral)."); 
		}
		
		//Add the emotion thought
		String emotionThought = "";
		
		if (!WordManager.exist(cuser, emotion, lang)) {
			emotionThought = WordManager.create(cuser, emotion, "", lang);
		} else {
			emotionThought = ThoughtManager.create(cuser, emotion, "", lang);
		}
		
		//Add the emotion
		NodeManager.oinsert(cuser, emotionThought, "/", "emotion", emotion, "STR");
		Random ran = new Random();
		NodeManager.oinsert(cuser, emotionThought, "/", "percent", ran.nextInt(100)+"", "NUM");
		
		String the_emotion = "";
		
		if (theLevel==2) {
			the_emotion = RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_l_?]"), LinkManager.get_value("@[th_fr_l]")+" "+emotionThought, "1 2", Start.FIRST_LANGUAGE, "L");
			RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_une_émotion]"), the_emotion+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[r_fr_une_émotion]"), "1 2 3", lang, "L");
			
		} else if (theLevel==1) {
			the_emotion = RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_le_?]"), LinkManager.get_value("@[th_fr_le]")+" "+emotionThought, "1 2", Start.FIRST_LANGUAGE, "L");
			RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_une_émotion]"), the_emotion+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[r_fr_une_émotion]"), "1 2 3", lang, "L");
			
		} else {
			the_emotion = RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_la_?]"), LinkManager.get_value("@[th_fr_la]")+" "+emotionThought, "1 2", Start.FIRST_LANGUAGE, "L");
			RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_une_émotion]"), the_emotion+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[r_fr_une_émotion]"), "1 2 3", lang, "L");
			
		}
		
		if (parentThought!=null && !parentThought.equals("")) {
			
			if (extreme) {
				
				if (parentAnLevel==1) RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_?]"), the_emotion+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[th_fr_un]")+" "+LinkManager.get_value("@[th_fr_extrême]")+" "+parentThought, "1 2 3 3 3", Start.FIRST_LANGUAGE, "L");
				else RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_?]"), the_emotion+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[th_fr_une]")+" "+LinkManager.get_value("@[th_fr_extrême]")+" "+parentThought, "1 2 3 3 3", Start.FIRST_LANGUAGE, "L");
				
			} else {

				if (parentAnLevel==1) RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_?]"), the_emotion+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[th_fr_un]")+" "+LinkManager.get_value("@[th_fr_léger]")+" "+parentThought, "1 2 3 3 3", Start.FIRST_LANGUAGE, "L");
				else RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_?]"), the_emotion+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[th_fr_une]")+" "+LinkManager.get_value("@[th_fr_légère]")+" "+parentThought, "1 2 3 3 3", Start.FIRST_LANGUAGE, "L");
				
			}
			
		} else {
			
			RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_?]"), the_emotion+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[r_fr_une_émotion_de_base]"), "1 2 3", Start.FIRST_LANGUAGE, "L");
			
		}
		
		return emotionThought;
				
	}

}
