/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.search;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.db.file.experience.ExperienceManagerL2;
import re.jpayet.mentdb.core.entity.circle.CircleManager;
import re.jpayet.mentdb.core.entity.relation.RelationManager;
import re.jpayet.mentdb.tools.Misc;

public class DeepSearchEngine {
	
	public static String allTypeList = "ASLGIOUDR";
	
	@SuppressWarnings("unchecked")
	public static JSONArray execute_from_mql(JSONArray relations, String typeFilter, String typeOrder, int nbsearch, int main_depth) throws Exception {
		
		//Initialization
		JSONArray result = new JSONArray();
		JSONObject exclusion = new JSONObject();

		typeFilter = typeFilter.replace(" ", "");
		typeOrder = typeOrder.replace(" ", "");
		
		if (typeFilter.equals("*")) typeFilter = DeepSearchEngine.allTypeList;
		if (typeOrder.equals("*")) typeOrder = DeepSearchEngine.allTypeList;
		
		for(int z=0;z<typeFilter.length();z++) {
			if (DeepSearchEngine.allTypeList.indexOf(""+typeFilter.charAt(z))==-1) {
				
				throw new Exception("Sorry, the type filter '"+typeFilter.charAt(z)+"' is not valid ("+DeepSearchEngine.allTypeList+").");
				
			}
		}
		
		for(int z=0;z<typeOrder.length();z++) {
			if (DeepSearchEngine.allTypeList.indexOf(""+typeOrder.charAt(z))==-1) {
				
				throw new Exception("Sorry, the type order '"+typeOrder.charAt(z)+"' is not valid ("+DeepSearchEngine.allTypeList+").");
				
			}
		}
		
		for(int i=0;i<nbsearch;i++) {

			int depth = main_depth;
			String[] relation_found = {null, null};
			JSONObject circle_exclusion = new JSONObject();
			
			//Search a relation
			//Parse all relations from a basic search
			for (int j=0;j<relations.size();j++) {
				
				String r_from_basic_search = Misc.atom(relations.get(j)+"", 1, " ");
				
				//Search in circle
				searchInCircle(r_from_basic_search, typeFilter, typeOrder, relation_found, exclusion, circle_exclusion, depth);
				
				if (relation_found[0]!=null) {
					break;
				}
				
				depth--;
				
				if (depth==0) {
					break;
				}
				
			}
			
			if (relation_found[0]==null) {
				
				break;
				
			} else {
				
				result.add(relation_found[0]+" "+relation_found[1]);
				exclusion.put(relation_found[0], 0);
				
			}
			
		}
		
		return result;
		
	}
	
	@SuppressWarnings("unchecked")
	public static JSONArray execute_from_brain(JSONArray relations, String typeFilter, String typeOrder, int nbsearch, int main_depth, JSONObject exclusion) throws Exception {
		
		//Initialization
		JSONArray result = new JSONArray();

		typeFilter = typeFilter.replace(" ", "");
		typeOrder = typeOrder.replace(" ", "");
		
		if (typeFilter.equals("*")) typeFilter = DeepSearchEngine.allTypeList;
		if (typeOrder.equals("*")) typeOrder = DeepSearchEngine.allTypeList;
		
		for(int i=0;i<nbsearch;i++) {

			int depth = main_depth;
			String[] relation_found = {null, null};
			JSONObject circle_exclusion = new JSONObject();
			
			//Search a relation
			//Parse all relations from a basic search
			for (int j=0;j<relations.size();j++) {
				
				String r_from_basic_search = Misc.atom(relations.get(j)+"", 1, " ");
				
				//Search in circle
				searchInCircle(r_from_basic_search, typeFilter, typeOrder, relation_found, exclusion, circle_exclusion, depth);
				
				if (relation_found[0]!=null) {
					break;
				}
				
				depth--;
				
				if (depth==0) {
					break;
				}
				
			}
			
			if (relation_found[0]==null) {
				
				break;
				
			} else {
				
				result.add(relation_found[0]+" "+relation_found[1]);
				exclusion.put(relation_found[0], 0);
				
			}
			
		}
		
		return result;
		
	}
	
	@SuppressWarnings("unchecked")
	public static void searchInCircle(String r, String typeFilter, String typeOrder, String[] relation_found, JSONObject exclusion, JSONObject circle_exclusion, int depth) throws Exception {
		
		JSONArray circle_ids = CircleManager.getIds(r);
		for(int i=0;i<circle_ids.size();i++) {
			
			String rc = (String) circle_ids.get(i);
			
			if (!circle_exclusion.containsKey(rc)) {
				checker(rc, typeFilter, typeOrder, relation_found, exclusion, circle_exclusion, depth);
				circle_exclusion.put(rc, 0);
			}
			
			if (relation_found[0]!=null) {
				return;
			}
			
		}
		
	}
	
	public static void checker(String r, String typeFilter, String typeOrder, String[] relation_found, JSONObject exclusion, JSONObject circle_exclusion, int depth) throws Exception {

		check_relation(r, typeFilter, typeOrder, relation_found, exclusion, depth);
		
		if (relation_found[0]==null) {
			
			String parent_id = check_parent(r, typeFilter, typeOrder, relation_found, exclusion, depth);
			
			if (relation_found[0]==null) {
				
				check_in_relation(r, typeFilter, typeOrder, relation_found, exclusion, circle_exclusion, depth);
				
				if (relation_found[0]==null) {
					
					check_sub_relation(r, typeFilter, typeOrder, relation_found, exclusion, circle_exclusion, depth);
					
					if (relation_found[0]==null && parent_id!=null) {
						
						searchInCircle(parent_id, typeFilter, typeOrder, relation_found, exclusion, circle_exclusion, depth-1);
						
					}
					
				}
				
			}
			
		}
		
	}
	
	public static void check_relation(String r, String typeFilter, String typeOrder, String[] relation_found, JSONObject exclusion, int depth) throws Exception {

		if (!exclusion.containsKey(r) && typeFilter.indexOf(r.charAt(1)+"")>-1) {
			relation_found[0] = r;
			relation_found[1] = depth+"";
		}
		
	}
	
	public static String check_parent(String r, String typeFilter, String typeOrder, String[] relation_found, JSONObject exclusion, int depth) throws Exception {

		JSONObject obj = Record.getNode(r);
		String parentId = (String) obj.get("p");
		
		if (parentId!=null && !exclusion.containsKey(parentId) && typeFilter.indexOf(parentId.charAt(1)+"")>-1) {
			relation_found[0] = parentId;
			relation_found[1] = depth+"";
		}
		
		return parentId;
		
	}
	
	public static void check_in_relation(String r, String typeFilter, String typeOrder, String[] relation_found, JSONObject exclusion, JSONObject circle_exclusion, int main_depth) throws Exception {

		//Search in branch
		for(int iBranch=0;iBranch<typeOrder.length();iBranch++) {
		
			String type = typeOrder.charAt(iBranch)+"";
			String tl = ExperienceManagerL2.getFirstTabLink("ai", r, type);
			
			int depth = main_depth;
			while(tl!=null && depth>0) {
				
				//Get the relation id
				String relationId = Misc.atom(tl, 1, " ").substring(1);
				
				//Search in circle
				searchInCircle(relationId, typeFilter, typeOrder, relation_found, exclusion, circle_exclusion, depth);
				
				if (relation_found[0]!=null) {
					return;
				}
				
				depth--;
				
				//Get the next down node
				tl = ExperienceManagerL2.getDownTabLink("ai", r, tl, type);
				
			}
			
			if (relation_found[0]!=null) {
				return;
			}
			
		}
		
	}
	
	public static void check_sub_relation(String r, String typeFilter, String typeOrder, String[] relation_found, JSONObject exclusion, JSONObject circle_exclusion, int main_depth) throws Exception {

		JSONArray tabLinkIds = RelationManager.listTabLinks("ai", r);
		for(int i=0;i<tabLinkIds.size();i++) {
			
			String currentRelationTabLink = ""+tabLinkIds.get(i);
			
			//Search in branch
			for(int iBranch=0;iBranch<typeOrder.length();iBranch++) {

				String type = typeOrder.charAt(iBranch)+"";
				String tl = ExperienceManagerL2.getFirstTabLink("ai", currentRelationTabLink, type);
				
				int depth = main_depth;
				while(tl!=null && depth>0) {
					
					//Get the relation id
					String relationId = Misc.atom(tl, 1, " ").substring(1);
					
					//Search in circle
					searchInCircle(relationId, typeFilter, typeOrder, relation_found, exclusion, circle_exclusion, depth);
					
					if (relation_found[0]!=null) {
						return;
					}
					
					depth--;
					
					//Get the next down node
					tl = ExperienceManagerL2.getDownTabLink("ai", currentRelationTabLink, tl, type);
					
				}
				
				if (relation_found[0]!=null) {
					return;
				}
				
			}
			
			if (relation_found[0]!=null) {
				return;
			}
		
		}
		
	}

}
