/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.session;

import java.net.*;
import java.io.*;

import re.jpayet.mentdb.core.command.Command;
import re.jpayet.mentdb.core.db.file.transaction.Transaction;
import re.jpayet.mentdb.core.session.env.EnvManager;
import re.jpayet.mentdb.core.stat.Statistic;
import re.jpayet.mentdb.server.Start;
import re.jpayet.mentdb.tools.Misc;

//The server threads
public class SessionThread extends Thread {

	//Initialization
	public Socket socket = null;
	public static long nbConnection = 0;
	public static long nbOpenConnection = 0;
	public long idConnection = 0;
	public long life = System.currentTimeMillis();
	public PrintWriter out = null;
	public BufferedReader in = null;
	public int nbExecution = 1;
	public String user = "";
	public String gridMenu = "BRANCH";
	public String graph_type = null;
	public String graph_obj = null;
	
	//The environment variable
	public EnvManager env = new EnvManager();

	//The constructor
	public SessionThread(Socket socket) {

		//Set the default values
		super("ServerThread");
		nbConnection++;
		Statistic.incrementNbSession();
		idConnection = nbConnection;
		this.socket = socket;

	}

	//Thread run method
	public void run() {
		
		//Increment the number of thread
		nbOpenConnection++;

		//Try to get streams
		try {
			
			//Out and in stream initialization
			out = new PrintWriter(socket.getOutputStream(), true);
			in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			
			//Initialization
			String inputLine;
			
			//Send the server version and others ... (the first data line)
			out.println("1MentDB v_"+Start.version+", Connection id: "+idConnection);

			//Read data from client
			while ((inputLine = in.readLine()) != null) {
				
				//Reset the life counter
				life=System.currentTimeMillis();

				//The result container
				String result = "";
				inputLine = inputLine.replace("[MENTDB_CRLF]", "\r\n").replace("[MENTDB_LF]", "\n").replace("[MENTDB_CR]", "\r");
				
				//Try to evaluate the request
				try {
					
					//Evaluate the request
					result = Command.executeAllCommands(true, this, Misc.splitCommand(inputLine), this.env);
					
					//Prepare the result
					if (result==null) {
						
						out.println("N");
					
					} else {
						
						//Reset the socket read timeout to 0 (infinite)
						if (nbExecution==1) {
							
							socket.setSoTimeout(0);
						
						}
						
						out.println("1"+result.replace("\r\n", "[MENTDB_CRLF]").replace("\n", "[MENTDB_LF]").replace("\r", "[MENTDB_CR]"));
					
					}

				} catch (Exception e) {
					
					//Protocol connection error
					if (nbExecution==1) {
						
						result = "jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht7";
						out.println("1"+result.replace("\r\n", "[MENTDB_CRLF]").replace("\n", "[MENTDB_LF]").replace("\r", "[MENTDB_CR]"));
					
					} else {
						
						//An error was generated
						if (e.getMessage()==null) out.println("0 unexpected error (Null returned).");
						else out.println("0"+(e.getMessage()+"").replace("\r\n", "[MENTDB_CRLF]").replace("\n", "[MENTDB_LF]").replace("\r", "[MENTDB_CR]"));
						
					}

				}

				//Break if the exit error code was returned
				if (result!=null && result.startsWith("jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht")) {
					break;
				}

				//Increment the number of execution
				nbExecution++;
				Statistic.incrementNbCommand();
				
				//Reset the life counter
				life=System.currentTimeMillis();

			}

			//Close the socket
			socket.close();

		} catch (Exception e) {
			
			//Nothing to do ...
			
		} finally {
			
			if (!user.equals("")) {
				
				try {
					
					Transaction.rollback(user);
					Misc.deleteFile("data"+File.separator+"transaction"+File.separator+user);
					Transaction.logFiles.remove(user);
					
				} catch (Exception e) {
					//e.printStackTrace();
				}
				
			}
			
			//Decrement the number of thread
			nbOpenConnection--;
			
			//Close out and in stream
			try {in.close();} catch (Exception e) {};
			try {out.close();} catch (Exception e) {};

		}
		
	}

}